@@ -136,7 +136,7 @@ incheck = alac.checkPath(input_path)
 if incheck == "existing_archive":
         print(just_table)
         table_path = "".join(input())
-        tp = alac.checkPath(table_path):
+        tp = alac.checkPath(table_path)
         if tp == "table":
                 tables = pickTable()
         elif tp == "overwrite_table":
@@ -149,49 +149,44 @@ if incheck == "existing_archive":
                 raise Exception("Invalid table output path!")
         ## settings flags will go here
         a = alac.config(input_path, table_path=table_path, tables=tables, GUI_mode = True)
-        wait()
         alac.parseTables(a)
 if incheck == "pdf_directory":
         print(both)
         next_path = "".join(input())
-        np = alac.checkPath(next_path):
-                if np == case "existing_archive":
-                        archive_path = next_path
-                        makeArchive = True
-                if np == "archive":
-                        archive_path = next_path
-                        makeArchive = True
-                if np == "overwrite_all_tables":
-                        makeArchive = False
-                        table_path = next_path
-                        makeAllTables = True
-                        a = alac.config(input_path, table_path=table_path, tables="all_tables")
-                        wait()
-                        alac.parseTables(a)
-                if np == "overwrite_table":
-                        makeArchive = False
-                        table_path = next_path
-                        makeTable = True
-                        tables = pickTable()
-                        a = alac.config(input_path, table_path=table_path, tables=tables)
-                        wait()
-                        alac.parseTables(a)
-                if np == "table":
-                        makeArchive = False
-                        makeTable = True
-                        table_path = next_path
-                        tables = pickTable()
-                        a = alac.config(input_path, table_path=table_path, tables=tables)
-                        wait()
-                        alac.parseTables(a)
-                if np == "all_tables":
-                        makeArchive = False
-                        makeAllTables = True
-                        tables = "all_tables"
-                        table_path = next_path
-                        a = alac.config(input_path, table_path=table_path, tables="all_tables")
-                        wait()
-                        alac.parseTables(a)
+        np = alac.checkPath(next_path)
+        if np == "existing_archive":
+                archive_path = next_path
+                makeArchive = True
+        if np == "archive":
+                archive_path = next_path
+                makeArchive = True
+        if np == "overwrite_all_tables":
+                makeArchive = False
+                table_path = next_path
+                makeAllTables = True
+                a = alac.config(input_path, table_path=table_path, tables="all_tables")
+                alac.parseTables(a)
+        if np == "overwrite_table":
+                makeArchive = False
+                table_path = next_path
+                makeTable = True
+                tables = pickTable()
+                a = alac.config(input_path, table_path=table_path, tables=tables)
+                alac.parseTables(a)
+        if np == "table":
+                makeArchive = False
+                makeTable = True
+                table_path = next_path
+                tables = pickTable()
+                a = alac.config(input_path, table_path=table_path, tables=tables)
+                alac.parseTables(a)
+        if np == "all_tables":
+                makeArchive = False
+                makeAllTables = True
+                tables = "all_tables"
+                table_path = next_path
+                a = alac.config(input_path, table_path=table_path, tables="all_tables")
+                alac.parseTables(a)
         if makeArchive:
                 print(just_table)
                 last_path = "".join(input())
@@ -201,38 +196,35 @@ if incheck == "pdf_directory":
                         makeAllTables = True
                         table_path = last_path
                         tables = "all_tables"
-                        wait()
                         alac.writeArchive(a)
                         b = alac.config(archive_path, table_path=table_path, tables="all_tables", GUI_mode=True, force_overwrite=True)
-                        alac.parseTables(b, tables)
+                        alac.parseTables(b)
                 elif tc == "overwrite_table":
                         makeTable = True
-                        tables = pickTable()
+                        if tables == "":
+                                tables = pickTable()
                         table_path = last_path
-                        wait()
                         alac.writeArchive(a)
-                        b = alac.config(archive_path, table_path=table_path, tables=table, GUI_mode=True, force_overwrite=True)
-                        alac.parseTables(b, tables)
+                        b = alac.config(archive_path, table_path=table_path, tables=tables, GUI_mode=True, force_overwrite=True)
+                        alac.parseTables(b)
                 elif tc == "table":
                         makeTable = True
-                        tables = pickTable()
+                        if tables == "":
+                                tables = pickTable()
                         table_path = last_path
-                        wait()
                         alac.writeArchive(a)
                         b = alac.config(archive_path, table_path=table_path, tables=tables, GUI_mode=True)
-                        alac.parseTables(b, tables)
+                        alac.parseTables(b)
                 elif tc == "all_tables":
                         makeAllTables = True
                         table_path = last_path
                         tables = "all_tables"
-                        wait()
                         alac.writeArchive(a)
                         b = alac.config(archive_path, table_path=table_path, tables="all_tables", GUI_mode=True)
-                        alac.parseTables(b, tables)
+                        alac.parseTables(b)
                 else:
                         makeTable = False
                         makeAllTables = False
-                        wait()
                         alac.writeArchive(a)
         if makeTable or makeAllTables:
                 if makeArchive:
@@ -242,7 +234,6 @@ if incheck == "pdf_directory":
                 else:
                         tables = "all_tables"
                 a = alac.config(input_path, table_path=table_path, tables=tables, GUI_mode = True)
-                wait()
                 alac.parseTables(a, tables)
 
 
